import { Application, Request, RequestHandler, Response, Router } from 'express';
import { Translations } from 'ooth-i18n';
import * as passport from 'passport';
export interface Backend {
    getUserById(id: string): Promise<User>;
    getUser(fields: {
        [key: string]: any;
    }): Promise<User>;
    getUserByValue(fields: string[], value: any): Promise<User>;
    updateUser(id: string, fields: {
        [key: string]: any;
    }): Promise<any>;
    insertUser(fields: {
        [key: string]: any;
    }): Promise<any>;
}
export declare type Options = {
    app: Application;
    backend: Backend;
    path?: string;
    sessionSecret?: string;
    session?: RequestHandler;
    defaultLanguage?: string;
    translations?: Translations;
};
export declare type StrategyValues = {
    [field: string]: any;
};
export declare type User = {
    _id: string;
    [strategyName: string]: undefined | string | Date | StrategyValues;
};
declare type Condition = (req: FullRequest) => boolean;
declare type Method<T> = (params: any, user: string | undefined, locale: string) => Promise<T>;
declare type Omit<T, K> = Pick<T, Exclude<keyof T, K>>;
declare type Override<T, K> = Omit<T, keyof K> & K;
export declare type FullRequest = Override<Request, {
    locale: string;
    user?: string | undefined;
    session?: {
        id: string;
        cookie: {
            maxAge?: number;
        };
    };
}>;
declare type MyRequestHandler = (req: FullRequest, res: Response, next: () => any) => any;
declare type UserIdResolver<T> = (req: FullRequest, authResult: T) => Promise<string | undefined>;
declare type Result = {
    [key: string]: any;
};
declare type Afterware = ((res: Result, userId: string | undefined, req: FullRequest, response: Response) => Promise<Result>);
declare type EventListener = (payload: any) => Promise<void>;
export declare class Ooth {
    private path;
    private uniqueFields;
    private strategies;
    private route;
    private translations;
    private defaultLanguage;
    private __;
    private backend;
    private app;
    private middleware;
    private afterware;
    private authAfterware;
    private session?;
    private listeners;
    constructor({ app, backend, path, defaultLanguage, translations, session, sessionSecret }: Options);
    usesSession(): boolean;
    registerUniqueField(strategyName: string, id: string, fieldName: string): void;
    registerStrategyUniqueField(strategyName: string, ...fieldNames: string[]): void;
    registerProfileFields(strategyName: string, ...fieldNames: string[]): void;
    getUniqueField: (user: User, fieldName: string) => any;
    getUserByFields: (strategyName: string, fields: {
        [key: string]: any;
    }) => Promise<User>;
    requireLogged: (req: Override<Request, {
        locale: string;
        user?: string | undefined;
        session?: {
            id: string;
            cookie: {
                maxAge?: number | undefined;
            };
        } | undefined;
    }>, res: Response, next: () => any) => import("express-serve-static-core").Response | undefined;
    requireNotLogged: (req: Override<Request, {
        locale: string;
        user?: string | undefined;
        session?: {
            id: string;
            cookie: {
                maxAge?: number | undefined;
            };
        } | undefined;
    }>, res: Response, next: () => any) => import("express-serve-static-core").Response | undefined;
    requireNotRegistered: (req: Override<Request, {
        locale: string;
        user?: string | undefined;
        session?: {
            id: string;
            cookie: {
                maxAge?: number | undefined;
            };
        } | undefined;
    }>, res: Response, next: () => any) => import("express-serve-static-core").Response | undefined;
    requireRegisteredWith: (strategy: string) => (req: Override<Request, {
        locale: string;
        user?: string | undefined;
        session?: {
            id: string;
            cookie: {
                maxAge?: number | undefined;
            };
        } | undefined;
    }>, res: Response, next: () => any) => import("express-serve-static-core").Response | undefined;
    registerPrimaryAuth<T>(strategyName: string, method: string, middleware: MyRequestHandler[], strategy: passport.Strategy, resolveUserId?: UserIdResolver<T>): void;
    registerPrimaryConnect(strategyName: string, method: string, middleware: MyRequestHandler[], strategy: passport.Strategy): void;
    registerSecondaryAuth(strategyName: string, method: string, condition: Condition, strategy: passport.Strategy): void;
    registerMethod<T>(strategyName: string, method: string, middleware: MyRequestHandler[], handler: Method<T>, httpMethod?: 'POST' | 'GET'): void;
    callMethod<T>(strategyName: string, method: string, params: any, user: any, locale: string): Promise<T>;
    registerMiddleware(...middleware: RequestHandler[]): void;
    registerAfterware(...afterware: Afterware[]): void;
    registerAuthAfterware(...afterware: Afterware[]): void;
    getUserById: (id: string) => Promise<User>;
    getUserByUniqueField: (fieldName: string, value: any) => Promise<User>;
    updateUser: (strategyName: string, id: string, fields: StrategyValues) => Promise<any>;
    getProfile(user: User | undefined): User | undefined;
    insertUser: (strategyName: string, fields: StrategyValues) => Promise<any>;
    getApp(): Application;
    getRoute(): Router;
    on(strategyName: string, eventName: string, eventListener: EventListener): void;
    emit(strategyName: string, eventName: string, payload: any): Promise<void>;
    private getStrategy;
}
export {};
